#ifndef DIM
#error DIM unset
#endif

#include <dune/common/fmatrix.hh>
#include <dune/common/fvector.hh>
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wignored-qualifiers"
#include <dune/grid/alugrid.hh>
#pragma clang diagnostic pop
#include <dune/istl/bvector.hh>
#include <dune/istl/bcrsmatrix.hh>

#include <dune/fufem/functionspacebases/p1nodalbasis.hh>

using SmallVector = Dune::FieldVector<double, DIM>;
using SmallMatrix = Dune::FieldMatrix<double, DIM, DIM>;
using Matrix = Dune::BCRSMatrix<SmallMatrix>;
using Vector = Dune::BlockVector<SmallVector>;

using Grid = Dune::ALUGrid<DIM, DIM, Dune::simplex, Dune::nonconforming>;
using GridView = Grid::LeafGridView;
using P1Basis = P1NodalBasis<GridView, double>;
using MyAssembler = Assembler<P1Basis, P1Basis>;

template void assembleNeumann<GridView, SmallVector, MyAssembler>(
    GridView const &gridView, MyAssembler const &assembler,
    Dune::BitSetVector<1> const &neumannNodes,
    Dune::BlockVector<SmallVector> &f,
    Dune::VirtualFunction<double, double> const &neumann, double relativeTime);

template Dune::shared_ptr<Dune::BlockVector<Dune::FieldVector<double, 1>>>
assembleFrictionWeightsal<GridView, SmallVector, MyAssembler>(
    GridView const &gridView, MyAssembler const &assembler,
    Dune::BitSetVector<1> const &frictionalNodes);

template Dune::shared_ptr<Dune::GlobalNonlinearity<Matrix, Vector>>
assembleNonlinearity<Matrix, Vector>(
    Dune::BitSetVector<1> const &frictionalNodes,
    Dune::BlockVector<Dune::FieldVector<double, 1>> const &nodalIntegrals,
    FrictionData const &fd);
