#ifndef RUINA_STATE_UPDATER_HH
#define RUINA_STATE_UPDATER_HH

#include "compute_state_ruina.hh"
#include "stateupdater.hh"

template <class SingletonVector, class Vector>
class RuinaStateUpdater : public StateUpdater<SingletonVector, Vector> {
public:
  RuinaStateUpdater(SingletonVector _logState_initial,
                    Dune::BitSetVector<1> const &_nodes, double _L);

  virtual void nextTimeStep();
  virtual void setup(double _tau);
  virtual void solve(Vector const &velocity_field);
  virtual void extractLogState(SingletonVector &);

private:
  SingletonVector logState_o;
  SingletonVector logState;
  Dune::BitSetVector<1> const &nodes;
  double const L;
  double tau;
};

template <class SingletonVector, class Vector>
RuinaStateUpdater<SingletonVector, Vector>::RuinaStateUpdater(
    SingletonVector _logState_initial, Dune::BitSetVector<1> const &_nodes,
    double _L)
    : logState(_logState_initial), nodes(_nodes), L(_L) {}

template <class SingletonVector, class Vector>
void RuinaStateUpdater<SingletonVector, Vector>::nextTimeStep() {
  logState_o = logState;
}

template <class SingletonVector, class Vector>
void RuinaStateUpdater<SingletonVector, Vector>::setup(double _tau) {
  tau = _tau;
}

template <class SingletonVector, class Vector>
void RuinaStateUpdater<SingletonVector, Vector>::solve(
    Vector const &velocity_field) {
  for (size_t i = 0; i < nodes.size(); ++i)
    if (nodes[i][0]) {
      double const V = velocity_field[i].two_norm();
      logState[i] = state_update_ruina(tau, V * tau / L, logState_o[i]);
    }
}

template <class SingletonVector, class Vector>
void RuinaStateUpdater<SingletonVector, Vector>::extractLogState(
    SingletonVector &_logState) {
  _logState = logState;
}

#endif
